/*
 * $Id$
 *
 *  MY Module.
 *
 * Copyright (C) 2006 Inode GmbH (Andreas Granig <andreas.granig@inode.info>)
 *
 * This file is part of Kamailio, a free SIP server.
 *
 * Kamailio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * Kamailio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *

 */
/*! \file
 * \brief MYMODULE
 *
 * \ingroup
 * - Module: mymodule
 */

#include <stdio.h>
#include "../../sr_module.h"
#include "../../dprint.h"
#include "../../rpc_lookup.h"
#include "../../ut.h"        /* str_init */
#include "../../globals.h"   /* is_main */
#include "../../lib/kcore/statistics.h"
#include "../../modules/sl/sl.h"
#include "../../mod_fix.h"
#include "../../socket_info.h"
#include "../../pvar.h"
#include "../../lib/kcore/km_ut.h"
#include "../../mem/memdbg.h"
#include "../../cfg/cfg.h" /* memlog */
#include <config.h>
#include "my_mod.h"




MODULE_VERSION  /*Module */

unsigned int init_my_flag= 0;

#define MY_USER_COL             "username"
#define CURR_IDN_COL            "currident"
#define PREV_IDN_COL            "prevident"
#define CURR_ID_COL             "currid"
#define PREV_ID_COL             "previd"
#define CURR_ADDR_COL           "curraddr"
#define PREV_ADDR_COL           "prevaddr"

static int mod_init(void);                          /*!< Module initialization function */
static void mod_destroy(void);                          /*!< Module destroy function */
static int child_init(int rank);                    /*!< Per-child init function */
static int w_mymodsave(struct sip_msg* _m, str* _d, char* username);
extern int bind_mymod(mymod_api_t* api);


str my_user_col          = str_init(MY_USER_COL);               /*!< Name of column containing usernames */
str curr_idn_col         = str_init(CURR_IDN_COL);
str prev_idn_col         = str_init(PREV_IDN_COL);
str curr_id              = str_init(CURR_ID_COL);
str prev_id              = str_init(PREV_ID_COL);
str curr_addr            = str_init(CURR_ADDR_COL);
str prev_addr            = str_init(PREV_ADDR_COL);
str db_url               = str_init(DEFAULT_DB_URL);         /*!< Database URL */
int db_mode              = 3;                                /*!< Database sync scheme: 0-no db, 1-write through, 2-write back, 3-only db */
int mymod_fetch_rows = 2000;

db1_con_t* mymod_dbh = 0; /* Database connection handle */
db_func_t mymod_dbf;


/*
 * Script commands we export.
 */
static cmd_export_t cmds[]={
        {"mymodsave",         (cmd_function)w_mymodsave,       2,    0, 0,
                        REQUEST_ROUTE},
        {"mm_bind_mymod",        (cmd_function)bind_mymod,        1, 0, 0, 0},
        {0,0,0,0,0}
};

/*
 * Script parameters
 */
static param_export_t mod_params[] = {
        {"my_user_column",         STR_PARAM, &my_user_col.s      },
        {"curr_idn_column",     STR_PARAM,    &curr_idn_col.s  },
        {"prev_idn_column",     STR_PARAM,    &prev_idn_col.s  },
        {"curr_id",             STR_PARAM,    &curr_id.s  },
        {"prev_id",             STR_PARAM,    &prev_id.s  },
        {"curr_addr",           STR_PARAM,    &curr_addr.s  },
        {"prev_addr",           STR_PARAM,    &prev_addr.s  },
        {"mymod_fetch_rows",    INT_PARAM,    &mymod_fetch_rows.s   },
        {"db_url",              STR_PARAM,    &db_url.s        },
        {"db_mode",             INT_PARAM,    &db_mode         },
        {0, 0, 0}
};

/*
 * Export the statistics we have
 */
static stat_export_t mod_stats[] = {
  {0,0,0}
};

struct module_exports exports= {
  "mymodule",	/* module's name */
  DEFAULT_DLFLAGS, /*!< dlopen flags */
  cmds,          /* exported functions */
  mod_params,    /* param exports */
  mod_stats,     /* exported statistics */
  mod_init,      /* module initialization function */
  0,             /* reply processing function FIXME Not sure when this is used */
  mod_destroy,   /* Destroy function */
  child_init     /* per-child init function */
};

/**
 * @return 0 to continue to load the OpenSER, -1 to stop the loading
 * and abort OpenSER.
 */
static int mod_init(void)
{


DBG("Inside MY Mod Init\n");
#ifdef STATISTICS
        /* register statistics */
        if (register_module_stats( exports.name, mod_stats)!=0 ) {
                LM_ERR("failed to register core statistics\n");
                DBG("failed to register core statistics\n");
                return -1;
        }
#endif

     /* Compute the lengths of string parameters */
     my_user_col.len  = strlen(my_user_col.s);
     curr_idn_col.len = strlen(curr_idn_col.s);
     prev_idn_col.len = strlen(prev_idn_col.s);
     curr_id.len      = strlen(curr_id.s);
     prev_id.len      = strlen(prev_id.s);
     curr_addr.len    = strlen(curr_addr.s);
     prev_addr.len    = strlen(prev_addr.s);
     db_url.len           = strlen(db_url.s);

     if(db_url.s== NULL)
     {
        LM_ERR("database url not set!\n");
        DBG("database url not set!\n");
        return -1;
     }

     /* Shall we use database ? */
     if (db_mode != NO_DB)
     { /* Yes */
       DBG("Inside DB check\n");
       DBG("DBURL is %s", db_url);

         if (db_bind_mod(&db_url, &mymod_dbf) < 0) { /* Find database module */
             LM_ERR("failed to bind database module\n");
             DBG("failed to bind database module\n");
             return -1;
         }
         if (!DB_CAPABILITY(mymod_dbf, DB_CAP_ALL)) {
              LM_ERR("database module does not implement all functions"
                     " needed by the module\n");
              DBG("database module does not implement all functions"
                     " needed by the module\n");
              return -1;
         }
         if(mymod_fetch_rows<=0) {
              LM_ERR("invalid fetch_rows number '%d'\n", mymod_fetch_rows);

             return -1;
         }
     }

     init_mymod_flag = 1;

     return(0);
}

/**
 * Called only once when OpenSER is shuting down to clean up module
 * resources.
 */
static void mod_destroy() {

     if (mymod_dbh) {
         mymod_dbf.close(mymod_dbh);
     }

  return;
}

/**
 * The rank will be o for the main process calling this function,
 * or 1 through n for each listener process. The rank can have a negative
 * value if it is a special process calling the child init function.
 * Other then the listeners, the rank will equal one of these values:
 * PROC_MAIN      0  Main ser process
 * PROC_TIMER    -1  Timer attendant process
 * PROC_FIFO     -2  FIFO attendant process
 * PROC_TCP_MAIN -4  TCP main process
 * PROC_UNIXSOCK -5  Unix domain socket server processes
 *
 * If this function returns a nonzero value the loading of OpenSER will
 * stop.
 */
static int child_init(int rank)
{
  int rtn = 0;

   /* we need connection from working SIP and TIMER and MAIN
    * processes only */
    if (rank<=0 && rank!=PROC_TIMER && rank!=PROC_MAIN) {
    rtn = 0;
    }

    if (rank==PROC_INIT || rank==PROC_MAIN || rank==PROC_TCP_MAIN) {
	return 0; /* do nothing for the main process */
    }

    mymod_dbh = mymod_dbf.init(&db_url); /* Get a database connection per child */
    if (!mymod_dbh) {
         DBG("child(%d): failed to connect to database\n", rank);
         LM_ERR("child(%d): failed to connect to database\n", rank);
         rtn = -1;
    }

   return(rtn);
}

/*! \brief
 * Wrapper to save(location)
 */
static int w_mymodsave(struct sip_msg* _m, str* _d, char* username)
{
        return mymodsave(_m, _d, username);
}



